/*******************************************************************************
                     Computación Paralela 2012 - FaMAF
                      Universidad Nacional de Córdoba

Ejercicio 3 - Traspuesta
========================
El siguiente código inicializa una matriz cuadrada con una gradiente de colores
y luego muestra dos imágenes lado a lado.
Queremos que la segunda imagen (a la derecha) tenga la traspuesta de la original.
Complete el kernel transpose y el resto de las líneas FALTA en el programa para
que funcione correctamente.

********************************************************************************/

#include <cuda.h>
#include <cutil.h>
#include <stdio.h>
#include <stdlib.h>

#include "colorramp.h"
#include "sdlstuff.h"

#define N 307
#define SIZE (N * N)


__global__ void gradient_1d(rgba32 * image, size_t size) {

    // indice del thread en el vector
    size_t idx = blockIdx.x * blockDim.x + threadIdx.x;

    if (idx < size) {
        // color del bloque [0..1]
        float color = (float) idx / (size - 1);
        // pasar a rgb
        rgba32 rgba = color_1d(color);
        // escribir a memoria
        image[idx] = rgba;
    }
}


// Traspone una matriz cuadrada n x n
__global__ void transpose(const rgba32 * src, size_t n, rgba32 * dst) {
    // FALTA: el kernel!
}


int main(int argc, char ** argv) {

    // inicializar sdl
    sdls_init(2 * N + 1, N);

    rgba32 * device_orig;
    rgba32 * device_trans;
    CUDA_SAFE_CALL(cudaMalloc(&device_orig, SIZE * sizeof(rgba32)));
    CUDA_SAFE_CALL(cudaMalloc(&device_trans, SIZE * sizeof(rgba32)));

    // pintamos el destino de blanco para ver errores
    CUDA_SAFE_CALL(cudaMemset(device_trans, 255, SIZE * sizeof(rgba32)));

    // inicializamos la imagen original con el kernel gradient_1d
    dim3 block_1d(256);
    dim3 grid_1d((SIZE + block_1d.x - 1) / block_1d.x);
    gradient_1d<<<grid_1d, block_1d>>>(device_orig, SIZE);
    CUT_CHECK_ERROR("Fallo al lanzar gradient_1d:");

    // FALTA: trasponer device_orig a device_trans con transpose

    // FALTA: esperar que los kernels terminen

    // pedir memoria del host para la imagen
    rgba32 * host_orig = (rgba32 *) malloc(SIZE * sizeof(rgba32));
    rgba32 * host_trans = (rgba32 *) malloc(SIZE * sizeof(rgba32));
    // FALTA: copiar imágenes al host!

    // dibujar en pantalla
    sdls_drawrectangle(0, 0, N, N, host_orig);
    sdls_drawrectangle(N + 1, 0, N, N, host_trans);

    printf("<ENTER> para salir\n");
    getchar();

    // limpieza
    free(host_orig);
    free(host_trans);
    CUDA_SAFE_CALL(cudaFree(device_orig));
    CUDA_SAFE_CALL(cudaFree(device_trans));

    return 0;
}
